// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:dotted_border/dotted_border.dart';
import 'package:flutter/material.dart' hide ButtonStyle;
import 'package:flutter/services.dart';
import 'package:gap/gap.dart';
import 'package:wyatt_bloc_helper/wyatt_bloc_helper.dart';
import 'package:wyatt_ui_components/wyatt_wyatt_ui_components.dart';
import 'package:wyatt_ui_kit/src/components/buttons/cubit/button_cubit.dart';
import 'package:wyatt_ui_kit/src/components/buttons/cubit/invalid_button_cubit.dart';
import 'package:wyatt_ui_kit/src/components/buttons/file_selection_button/file_selection_button_theme_resolver.dart';
import 'package:wyatt_ui_kit/src/components/gradients/gradient_text.dart';
import 'package:wyatt_ui_kit/src/core/helpers/linear_gradient_helper.dart';

class FileSelectionButtonScreen
    extends CubitScreen<InvalidButtonCubit, ButtonState> {
  const FileSelectionButtonScreen({
    this.leading,
    this.title,
    this.subTitle,
    this.disabledStyle,
    this.normalStyle,
    this.hoveredStyle,
    this.focusedStyle,
    this.tappedStyle,
    this.selectedStyle,
    this.invalidStyle,
    this.onPressed,
    this.mainAxisSize,
    this.themeResolver,
    super.key,
  });

  final Widget? leading;
  final TextWrapper? title;
  final TextWrapper? subTitle;
  final MainAxisSize? mainAxisSize;

  final FileSelectionButtonStyle? disabledStyle;
  final FileSelectionButtonStyle? normalStyle;
  final FileSelectionButtonStyle? hoveredStyle;
  final FileSelectionButtonStyle? focusedStyle;
  final FileSelectionButtonStyle? tappedStyle;
  final FileSelectionButtonStyle? selectedStyle;
  final FileSelectionButtonStyle? invalidStyle;

  final void Function(ControlState state)? onPressed;
  final FileSelectionButtonThemeResolver? themeResolver;

  @override
  InvalidButtonCubit create(BuildContext context) => InvalidButtonCubit();

  /// Negotiate the theme to get a complete style.
  FileSelectionButtonStyle resolve(BuildContext context, ButtonState state) {
    final FileSelectionButtonThemeResolver resolver = themeResolver ??
        FileSelectionButtonThemeResolver(
          computeExtensionValueFn: (
            context,
            defaultValue,
            themeExtension, {
            extra,
          }) {
            FileSelectionButtonStyle? style = defaultValue;
            switch (extra?.state) {
              case ControlState.disabled:
                style = themeExtension.disabledStyle;
                break;
              case ControlState.focused:
                style = themeExtension.focusedStyle;
                break;
              case ControlState.hovered:
                style = themeExtension.hoveredStyle;
                break;
              case ControlState.tapped:
                style = themeExtension.tappedStyle;
                break;
              case ControlState.normal:
              case null:
                style = themeExtension.normalStyle;
                break;
            }
            if (extra?.isSelected ?? false) {
              style = themeExtension.selectedStyle;
            }
            if (extra?.isInvalid ?? false) {
              style = themeExtension.invalidStyle;
            }

            return style;
          },
          customStyleFn: (context, {extra}) {
            FileSelectionButtonStyle? style;
            switch (extra?.state) {
              case ControlState.disabled:
                style = disabledStyle;
                break;
              case ControlState.focused:
                style = focusedStyle;
                break;
              case ControlState.hovered:
                style = hoveredStyle;
                break;
              case ControlState.tapped:
                style = tappedStyle;
                break;
              case ControlState.normal:
              case null:
                style = normalStyle;
                break;
            }
            if (extra?.isSelected ?? false) {
              style = selectedStyle;
            }
            if (extra?.isInvalid ?? false) {
              style = invalidStyle;
            }

            return style;
          },
        );
    return resolver.negotiate(context, extra: state);
  }

  Widget _border(
    BuildContext context,
    FileSelectionButtonStyle style,
    Widget child,
  ) {
    if (style.borderColors != null && style.stroke != null) {
      return DottedBorder(
        padding: EdgeInsets.zero,
        dashPattern: const [5, 5],
        strokeWidth: style.stroke!,
        color: style.borderColors!.color,
        borderType: BorderType.RRect,
        radius:
            style.radius?.resolve(TextDirection.ltr).bottomLeft ?? Radius.zero,
        strokeCap: StrokeCap.square,
        child: child,
      );
    } else {
      return child;
    }
  }

  @override
  Widget onBuild(BuildContext context, ButtonState state) {
    final style = resolve(context, state);

    return Focus(
      onFocusChange: (hasFocus) =>
          hasFocus ? bloc(context).onFocus() : bloc(context).onUnfocus(),
      onKeyEvent: (focusNode, event) {
        if (event.logicalKey == LogicalKeyboardKey.enter && state.isFocused) {
          onPressed?.call(state.state);
          bloc(context).onClickUpOut();
          return KeyEventResult.handled;
        }
        return KeyEventResult.ignored;
      },
      child: MouseRegion(
        cursor: SystemMouseCursors.click,
        onEnter: (event) {
          bloc(context).onMouseEnter();
        },
        onExit: (event) {
          bloc(context).onMouseLeave();
        },
        child: GestureDetector(
          onTapDown: (details) {
            bloc(context).onClickDown();
          },
          onTapUp: (details) {
            onPressed?.call(state.state);
            bloc(context).onClickUpIn();
          },
          onTapCancel: () {
            onPressed?.call(state.state);
            bloc(context).onClickUpOut();
          },
          child: _border(
            context,
            style,
            DecoratedBox(
              decoration: BoxDecoration(
                color: style.backgroundColors?.color,
                // if no gradient colors => no default value
                gradient: (style.backgroundColors?.isGradient ?? false)
                    ? LinearGradient(
                        colors: style.backgroundColors!.colors,
                      )
                    : null,
                boxShadow: [
                  if (style.shadow != null) ...[style.shadow!]
                ],
                borderRadius: style.radius,
              ),
              child: ConstrainedBox(
                constraints: const BoxConstraints(
                  minWidth: 200,
                  minHeight: 50,
                ), // min sizes for Material buttons
                child: Padding(
                  padding: style.padding ?? EdgeInsets.zero,
                  child: Row(
                    mainAxisSize: mainAxisSize ?? MainAxisSize.min,
                    children: [
                      if (leading != null) ...[
                        leading ?? const SizedBox.shrink(),
                        Gap((style.padding?.horizontal ?? 10)/2),
                      ],
                      Column(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          /// Choose color
                          /// title.style.color ??
                          ///  buttonStyle.title.style.color ??
                          ///    context.textTheme.titleLarge.color
                          ///
                          /// Choose gradient
                          /// title.gradient ??
                          ///  buttonStyle.foregroundColor.colors ??
                          ///    null
                          ///
                          /// More infos in `negociate()` method
                          if (title != null) ...[
                            Text(
                              title!.text,
                              style: title!.style ?? style.title,
                            ).toGradient(
                              LinearGradientHelper.fromNullableColors(
                                title?.gradient ??
                                    ((style.foregroundColors?.isGradient ??
                                            false)
                                        ? style.foregroundColors?.colors
                                        : null),
                              ),
                            ),
                          ],

                          /// Choose color
                          /// subTitle.style.color ??
                          ///  buttonStyle.subTitle.style.color ??
                          ///    context.textTheme.subTitleLarge.color
                          ///
                          /// Choose gradient
                          /// subTitle.gradient ??
                          ///  buttonStyle.foregroundColor.colors ??
                          ///    null
                          ///
                          /// More infos in `negociate()` method
                          if (subTitle != null) ...[
                            const Gap(5),
                            Text(
                              subTitle!.text,
                              style: subTitle!.style ?? style.subTitle,
                            ).toGradient(
                              LinearGradientHelper.fromNullableColors(
                                subTitle?.gradient ??
                                    ((style.foregroundColors?.isGradient ??
                                            false)
                                        ? style.foregroundColors?.colors
                                        : null),
                              ),
                            ),
                          ],
                        ],
                      ),
                    ],
                  ),
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }
}
